@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
-
-from typing import Any, Dict, Iterable, List, Tuple, Union
+from __future__ import annotations
+from typing import Any, Dict, Iterable, List, Optional, Tuple, Union
 
 import requests
 
@@ -9,12 +9,13 @@ from .base import Base
 from .document import Document, AttachmentDocument, extract_document_id_and_rev, SecurityDocument, \
     SecurityDocumentElement
 from .exceptions import CouchDBError, NameComplianceError
-from .utils import validate_db_name, content_type_getter, DEFAULT_TIMEOUT
+from .utils import validate_db_name, content_type_getter, DEFAULT_TIMEOUT, partitioned_db_resource_parser
 from .view import ViewResult
 
 
 __all__ = [
-    "Database"
+    "Database",
+    "Partition",
 ]
 
 
@@ -100,7 +101,7 @@ class Database(Base):
             **kwargs
     ) -> ViewResult:
         """
-        Executes the built-in _all_docs view, returning all of the documents in the database.
+        Executes the built-in _all_docs view, returning all the documents in the database (or partition).
 
         Parameters
         ----------
@@ -165,7 +166,7 @@ class Database(Base):
     def bulk_get(
             self,
             docs: List[Union[Dict, Document]],
-            revs: bool = False
+            revs: bool = False,
     ) -> List[Dict]:
         """
         This method can be called to query several documents in bulk. It is well suited for fetching a specific
@@ -465,7 +466,8 @@ class Database(Base):
             bookmark: str = None,
             update: bool = True,
             stable: bool = None,
-            execution_stats: bool = False
+            execution_stats: bool = False,
+            partition: str = None,
     ) -> Dict:
         """
         Find documents using a declarative JSON querying syntax.
@@ -509,6 +511,8 @@ class Database(Base):
         execution_stats : bool
             Include [execution statistics](https://docs.couchdb.org/en/main/api/database/find.html#find-statistics) in
             the query response. Default is `False`.
+        partition: str
+            An optional partition ID. Only valid for partitioned databases. (Default `None`.)
 
         Returns
         -------
@@ -519,7 +523,10 @@ class Database(Base):
           - `warning`
         """
         return self._post(
-            resource="_find",
+            resource=partitioned_db_resource_parser(
+                resource="_find",
+                partition=partition,
+            ),
             body={
                 "selector": selector,
                 "limit": limit,
@@ -689,7 +696,10 @@ class Database(Base):
         -------
         Document : A `couchdb3.document.Document` object containing the design document's content.
         """
-        return self.get(docid=f"_design/{ddoc}", **kwargs)
+        return self.get(
+            docid=f"_design/{ddoc}",
+            **kwargs
+        )
 
     def purge(
             self,
@@ -795,6 +805,7 @@ class Database(Base):
             validate_doc_update: str = None,
             views: Dict = None,
             autoupdate: bool = None,
+            partitioned: bool = None,
             **kwargs
     ) -> Tuple[str, bool, str]:
         """
@@ -822,6 +833,8 @@ class Database(Base):
             [View functions](https://docs.couchdb.org/en/latest/ddocs/ddocs.html#viewfun) definition.
         autoupdate : bool
             Indicates whether to automatically build indexes defined in this design document.
+        partitioned : bool
+            Set to `True` for a partitioned design.
         kwargs
         Further `Database.save` parameters.
 
@@ -829,6 +842,10 @@ class Database(Base):
         -------
         Tuple[str, bool, str] : The document's id ( `str`), the operation status (`bool`) and the revision ( `str`).
         """
+        if partitioned:
+            options = (options or dict()).update({
+                "partitioned": partitioned
+            })
         return self.save(
             doc={
                 "_id": f"_design/{ddoc}",
@@ -987,6 +1004,7 @@ class Database(Base):
             ddoc: str,
             view: str = None,
             *,
+            partition: str = None,
             conflicts: bool = None,
             descending: bool = None,
             endkey: str = None,
@@ -1010,7 +1028,295 @@ class Database(Base):
             update_seq: bool = None
     ) -> ViewResult:
         """
-        Executes the specified view function from the specified design documen, c.f [the official
+        Executes the specified view function from the specified design document, c.f [the official
+        documentation](https://docs.couchdb.org/en/main/api/ddoc/views.html#db-design-design-doc-view-view-name).
+
+        Parameters
+        ----------
+        ddoc : str
+            The corresponding design document's id.
+        view : str
+            The view's id.
+        partition: str
+            An optional partition ID. Only valid for partitioned databases. (Default `None`.)
+        conflicts : bool
+            Include conflicts information in response. Ignored if `include_docs` isn’t `True`. Default is `None`.
+        descending : bool
+            Return the documents in descending order by key. Default is `None`.
+        endkey : str
+             Stop returning records when the specified key is reached. Default is `None`
+        endkey_docid: str
+            Stop returning records when the specified document ID is reached. Ignored if `endkey` is not set. Default
+            is `None`.
+        group: bool
+            Group the results using the reduce function to a group or single row. Implies `reduce` is `true` and the
+            maximum `group_level`. Default is `None`.
+        group_level : int
+             Specify the group level to be used. Implies group is true. Default is `None`.
+        include_docs : bool
+            Include the associated document with each row. Default is `None`.
+        attachments : bool
+            Include the Base64-encoded content of attachments in the documents that are included if `include_docs` is
+            `True`. Ignored if `include_docs` isn’t `True`. Default is `None`.
+        att_encoding_info : bool
+            Include encoding information in attachment stubs if `include_docs` is `True` and the particular attachment
+            is compressed. Ignored if `include_docs` isn’t `True`. Default is `False`.
+        inclusive_end : bool
+            Specifies whether the specified end key should be included in the result. Default is `None`.
+        key : str
+            Return only documents that match the specified key. Default is `None`.
+        keys: Iterable[str]
+            Return only documents where the key matches one of the keys specified in the argument. Default is `None`.
+        limit : int
+            Limit the number of the returned documents to the specified number.  Default is `None`.
+        reduce : bool
+            Use the reduction function. Default is `True` when a reduce function is defined. Default is `None`.
+        skip : int
+            Skip this number of records before starting to return the results. Default is `None`.
+        sort : bool
+            Sort returned rows (see Sorting Returned Rows). Setting this to `False` offers a performance boost. The
+            `total_rows` and `offset` fields are not available when this is set to `False`. Default is `None`.
+        stable : bool
+            Whether or not the view results should be returned from a stable set of shards. Default is `None`.
+        startkey : str
+            Return records starting with the specified key. Default is `None`
+        startkey_docid : str
+            Return records starting with the specified document ID. Ignored if `startkey` is not set. Default is `None`
+        update : str
+            Whether or not the view in question should be updated prior to responding to the user. Supported values:
+
+            - `true`
+            - `false`
+            - `lazy`
+
+            Default is `None`.
+        update_seq : bool
+             Whether to include in the response an `update_seq` value indicating the sequence id of the database the
+             view reflects. Default is `False`.
+
+        Returns
+        -------
+        `view.ViewResult`
+        """
+        path = partitioned_db_resource_parser(
+            resource="_design",
+            partition=partition,
+        )
+        return ViewResult(**self._get(
+            resource=f"{path}/{ddoc}/_view/{view}" if (ddoc and view) else ddoc,
+            query_kwargs={
+                "conflicts": conflicts,
+                "descending": descending,
+                "endkey": endkey,
+                "endkey_docid": endkey_docid,
+                "group": group,
+                "group_level": group_level,
+                "include_docs": include_docs,
+                "attachments": attachments,
+                "att_encoding_info": att_encoding_info,
+                "inclusive_end": inclusive_end,
+                "key": key,
+                "keys": keys,
+                "limit": limit,
+                "reduce": reduce,
+                "skip": skip,
+                "sorted": sort,
+                "stable": stable,
+                "startkey": startkey,
+                "startkey_docid": startkey_docid,
+                "update": update,
+                "update_seq": update_seq
+            }
+        ).json())
+
+    def get_partition(self, partition_id: str) -> Partition:
+        """
+        Get a given partition.
+
+        Parameters
+        ----------
+        partition_id : str
+            The partition's ID.
+
+        Returns
+        -------
+        `Partition`
+        """
+        return Partition(
+            partition_id=partition_id,
+            name=self.name,
+            url=self.url,
+            port=self.port,
+            user=self._user,
+            password=self._password,
+            disable_ssl_verification=not self.session.verify,
+            auth_method=self.auth_method,
+            session=self.session,
+        )
+
+
+class Partition(Database):
+    """
+    Abstract Couchdb partition
+    """
+    def __init__(
+            self,
+            partition_id: str,
+            name: str,
+            *,
+            url: str = None,
+            port: int = None,
+            user: str = None,
+            password: str = None,
+            disable_ssl_verification: bool = False,
+            auth_method: str = None,
+            timeout: int = DEFAULT_TIMEOUT,
+            session: requests.Session = None,
+    ) -> None:
+        """
+
+        Parameters
+        ----------
+        name : str
+            The name of the database.
+        url : str
+            The url of the CouchDB server formatted as `scheme://user:password@host:port`. For example:
+
+                "http://user:password@127.0.0.1:5984"
+                "https://couchdb.example.com"
+        port : int
+            The port of the CouchDB server. Can also be supplied via the url.
+        user : str
+            The CouchDB admin username. Can also be supplied via the url.
+        password : str
+            The CouchDB admin password. Can also be supplied via the url.
+        disable_ssl_verification : bool
+            Controls whether to verify the server’s TLS certificate. Set to `True` when connecting to a server with
+            self-signed TLS certificates. Default `False`.
+        auth_method : str
+            Authentication method. Choices are `cookie` or `basic`. Default is `couchdb3.utils.DEFAULT_AUTH_METHOD`.
+        timeout : int
+            The default timeout for requests. Default c.f. `couchdb3.utils.DEFAULT_TIMEOUT`.
+        session: requests.Session
+            A specific session to use. Optional - if not provided, a new session will be initialized.
+        """
+        super(Partition, self).__init__(
+            name=name,
+            url=url,
+            session=session,
+            port=port,
+            user=user,
+            password=password,
+            disable_ssl_verification=disable_ssl_verification,
+            auth_method=auth_method,
+            timeout=timeout
+        )
+        self.partition_id = partition_id
+        # self.root = f"{name}/_partition/{partition_id}"
+
+    def __repr__(self) -> str:
+        return f"{super(Partition, self).__repr__()}/{self.partition_id}"
+
+    def all_docs(
+            self,
+            keys: Iterable[str] = None,
+            **kwargs
+    ) -> ViewResult:
+        """
+        Executes the built-in _all_docs view, returning all the documents in the partition.
+
+        Parameters
+        ----------
+        keys : Iterable[str]
+            Return only documents where the key matches one of the keys specified in the argument. Default is `None`.
+        kwargs
+            Further `couchdb3.database.Database.view` parameters.
+
+        Returns
+        -------
+        ViewResult
+        """
+        return super(Partition, self).all_docs(partition=self.partition_id, keys=keys, **kwargs)
+
+    # noinspection PyMethodOverriding
+    def info(
+            self,
+    ) -> Dict:
+        """
+        Return the partition's info by sending a `GET` request to `/self.root`.
+
+        Returns
+        -------
+        Dict: A dictionary containing the server's or database's info.
+        """
+        return super(Partition, self).info(partition=self.partition_id)
+
+    # noinspection PyMethodOverriding
+    def find(
+            self,
+            selector: Dict,
+            limit: int = 25,
+            skip: int = 0,
+            sort: List[Dict] = None,
+            fields: List[str] = None,
+            use_index: Union[str, List[str]] = None,
+            conflicts: bool = False,
+            r: int = 1,
+            bookmark: str = None,
+            update: bool = True,
+            stable: bool = None,
+            execution_stats: bool = False
+    ) -> Dict:
+        """
+        See `Database.find`.
+        """
+        return super(Partition, self).find(
+            selector=selector,
+            limit=limit,
+            skip=skip,
+            sort=sort,
+            fields=fields,
+            use_index=use_index,
+            conflicts=conflicts,
+            r=r,
+            bookmark=bookmark,
+            update=update,
+            stable=stable,
+            execution_stats=execution_stats,
+            partition=self.partition_id,
+        )
+
+    # noinspection PyMethodOverriding
+    def view(
+            self,
+            ddoc: str,
+            view: str = None,
+            *,
+            # partition: str = None,
+            conflicts: bool = None,
+            descending: bool = None,
+            endkey: str = None,
+            endkey_docid: str = None,
+            group: bool = None,
+            group_level: int = None,
+            include_docs: bool = None,
+            attachments: bool = None,
+            att_encoding_info: bool = None,
+            inclusive_end: bool = None,
+            key: str = None,
+            keys: Iterable[str] = None,
+            limit: int = None,
+            reduce: bool = None,
+            skip: int = None,
+            sort: bool = None,
+            stable: bool = None,
+            startkey: str = None,
+            startkey_docid: str = None,
+            update: str = None,
+            update_seq: bool = None
+    ) -> ViewResult:
+        """
+        Executes the specified view function from the specified design document, c.f [the official
         documentation](https://docs.couchdb.org/en/main/api/ddoc/views.html#db-design-design-doc-view-view-name).
 
         Parameters
@@ -1078,29 +1384,272 @@ class Database(Base):
         -------
         `view.ViewResult`
         """
-        return ViewResult(**self._get(
-            resource=f"_design/{ddoc}/_view/{view}" if (ddoc and view) else ddoc,
-            query_kwargs={
-                "conflicts": conflicts,
-                "descending": descending,
-                "endkey": endkey,
-                "endkey_docid": endkey_docid,
-                "group": group,
-                "group_level": group_level,
-                "include_docs": include_docs,
-                "attachments": attachments,
-                "att_encoding_info": att_encoding_info,
-                "inclusive_end": inclusive_end,
-                "key": key,
-                "keys": keys,
-                "limit": limit,
-                "reduce": reduce,
-                "skip": skip,
-                "sorted": sort,
-                "stable": stable,
-                "startkey": startkey,
-                "startkey_docid": startkey_docid,
-                "update": update,
-                "update_seq": update_seq
-            }
-        ).json())
+        return super(Partition, self).view(
+            ddoc=ddoc,
+            view=view,
+            partition=self.partition_id,
+            conflicts=conflicts,
+            descending=descending,
+            endkey=endkey,
+            endkey_docid=endkey_docid,
+            group=group,
+            group_level=group_level,
+            include_docs=include_docs,
+            attachments=attachments,
+            att_encoding_info=att_encoding_info,
+            inclusive_end=inclusive_end,
+            key=key,
+            keys=keys,
+            limit=limit,
+            reduce=reduce,
+            skip=skip,
+            sort=sort,
+            stable=stable,
+            startkey=startkey,
+            startkey_docid=startkey_docid,
+            update=update,
+            update_seq=update_seq
+        )
+
+    def bulk_docs(
+            self,
+            docs: List[Union[Dict, Document]],
+            new_edits: bool = True
+    ) -> List[Dict]:
+        """
+        See `Database.bulk_docs`.
+
+        Note:
+        Appends the partition's ID to the documents' ID.
+        """
+        return super(Partition, self).bulk_docs(
+            docs=[self.add_partition_to_doc(doc) for doc in docs],
+            new_edits=new_edits,
+        )
+
+    def bulk_get(
+            self,
+            docs: List[Union[Dict, Document]],
+            revs: bool = False,
+    ) -> List[Dict]:
+        """
+        See `Database.bulk_get`.
+
+        Note:
+        Appends the partition's ID to the documents' ID.
+        """
+        return super(Partition, self).bulk_get(
+            docs=[self.add_partition_to_doc(doc) for doc in docs],
+            revs=revs,
+        )
+
+    def copy(
+            self,
+            docid: str,
+            destid: str,
+            rev: str = None,
+            destrev: str = None
+    ) -> Tuple[str, bool, str]:
+        """
+        See `Database.copy`.
+
+        Note:
+        Appends the partition's ID to the document's ID.
+        """
+        return super(Partition, self).copy(
+            docid=self.add_partition_to_str(docid),
+            destid=self.add_partition_to_str(destid),
+            rev=rev,
+            destrev=destrev,
+        )
+
+    def create(
+            self,
+            doc: Union[Dict, Document],
+            *,
+            batch: bool = None
+    ) -> Tuple[str, bool, str]:
+        """
+        See `Database.create`.
+
+        Note:
+        Appends the partition's ID to the document's ID.
+        """
+        return super(Partition, self).create(
+            doc=self.add_partition_to_doc(doc),
+            batch=batch,
+        )
+
+    def delete(
+            self,
+            docid: str,
+            rev: str,
+            *,
+            batch: bool = None
+    ) -> bool:
+        """
+        See `Database.delete`.
+
+        Note:
+        Appends the partition's ID to the document's ID.
+        """
+        return super(Partition, self).delete(
+            docid=self.add_partition_to_str(docid),
+            rev=rev,
+            batch=batch,
+        )
+
+    def delete_attachment(
+            self,
+            docid: str,
+            attname: str,
+            rev: str,
+            *,
+            batch: bool = False
+    ) -> bool:
+        """
+        See `Database.delete_attachment`.
+
+        Note:
+        Appends the partition's ID to the document's ID.
+        """
+        return super(Partition, self).delete_attachment(
+            docid=self.add_partition_to_str(docid),
+            attname=attname,
+            rev=rev,
+            batch=batch,
+        )
+
+    def get(
+            self,
+            docid: str,
+            *,
+            attachments: bool = None,
+            att_encoding_info: bool = None,
+            atts_since: Iterable[str] = None,
+            conflicts: bool = None,
+            deleted_conflicts: bool = None,
+            latest: bool = None,
+            local_seq: bool = None,
+            meta: bool = None,
+            open_revs: Iterable[str] = None,
+            rev: str = None,
+            revs: bool = None,
+            revs_info: bool = None,
+            check: bool = False,
+            default_value: Any = None,
+    ) -> Union[Document, Any]:
+        """
+        See `Database.get`.
+
+        Note:
+        Appends the partition's ID to the document's ID.
+        """
+        return super(Partition, self).get(
+            docid=self.add_partition_to_str(docid),
+            attachments=attachments,
+            att_encoding_info=att_encoding_info,
+            atts_since=atts_since,
+            conflicts=conflicts,
+            deleted_conflicts=deleted_conflicts,
+            latest=latest,
+            local_seq=local_seq,
+            meta=meta,
+            open_revs=open_revs,
+            rev=rev,
+            revs=revs,
+            revs_info=revs_info,
+            check=check,
+            default_value=default_value,
+        )
+
+    def get_attachment(
+            self,
+            docid: str,
+            attname: str,
+            rev: str = None
+    ) -> AttachmentDocument:
+        """
+        See `Database.get_attachment`.
+
+        Note:
+        Appends the partition's ID to the document's ID.
+        """
+        return super(Partition, self).get_attachment(
+            docid=self.add_partition_to_str(docid),
+            attname=attname,
+            rev=rev,
+        )
+
+    def put_attachment(
+            self,
+            docid: str,
+            attname: str,
+            path: str = None,
+            *,
+            content: Any = None,
+            rev: str = None,
+    ) -> Tuple[str, bool, str]:
+        """
+        See `Database.put_attachment`.
+
+        Note:
+        Appends the partition's ID to the document's ID.
+        """
+        return super(Partition, self).put_attachment(
+            docid=self.add_partition_to_str(docid),
+            attname=attname,
+            path=path,
+            content=content,
+            rev=rev,
+        )
+
+    def rev(
+            self,
+            resource: str
+    ) -> Optional[str]:
+        """
+        See `Database.rev`.
+        """
+        return super(Partition, self).rev(self.add_partition_to_str(resource))
+
+    def save(
+            self,
+            doc: Union[Dict, Document],
+            batch: bool = None,
+            new_edits: bool = None,
+            path: str = None
+    ) -> Tuple[str, bool, str]:
+        """
+        See `Database.save`.
+
+        Note:
+        Appends the partition's ID to the document's ID.
+        """
+        return super(Partition, self).save(
+            doc=self.add_partition_to_doc(doc),
+            batch=batch,
+            new_edits=new_edits,
+            path=path,
+        )
+
+    def __contains__(self, item):
+        return super(Partition, self).__contains__(self.add_partition_to_str(item))
+
+    def add_partition_to_str(self, string: str) -> str:
+        """
+        Append the instance's partition ID to a string.
+        """
+        if string.startswith(self.partition_id):
+            return string
+        return f"{self.partition_id}:{string}"
+
+    def add_partition_to_doc(self, doc: Union[Document, Dict]) -> Union[Document, Dict]:
+        """
+        Append the instance's partition ID to the document's ID.
+        """
+        docid = doc.get("_id")
+        if docid is None:
+            return doc
+        doc["_id"] = self.add_partition_to_str(docid)
+        return doc
